﻿AceTargetLog		= AceAddonClass:new({
	name			= "AceTargetLog",
	description		= "Shows your last targeted and moused-over units in a nice list",
	version			= "2",
	aceCompatible	= "103",
	author			= "Silverwind",
	website			= "http://silverwind.wowinterface.com/",
	category		= "combat",
	db				= AceDatabase:new("AceTargetLogDB"),
	cmd				= AceChatCmdClass:new({"/acetargetlog","/atl"},
		{
			{
				option = "lock", desc = "Lock or unlock the frame", method = "LockFrame"
			},
			{
				option = "bgcolor", desc = "Set the Background color <red> <green> <blue> <alpha>", method = "SetBGColor", input = TRUE
			},
			{
				option = "show", desc = "Show the frame", method = "ShowFrame"
			},
			{
				option = "hide", desc = "Hide the frame", method = "HideFrame"
			}
			
		}
	)
})

local UnitTable = {}
local locked
local ListColors = {
	EnemyPC		= "|cffff0000",
	FriendlyPC	= "|cff11ff11",
	EnemyNPC	= "|cffff0000",
	FriendlyNPC	= "|cff44af44",
	NeutralNPC	= "|cffffff44",
	Party		= "|cff99ff99",
	Guild		= "|cff4444ff",
}
	
local ClassColors = GetLocale() == "deDE" and {
	["J\195\164ger"]		=	"|cffaad372",
	["Hexenmeister"]		=	"|cff9382C9",
	["Priester"]			=	"|cffffffff",
	["Paladin"]				=	"|cfff48cba",
	["Magier"]				=	"|cff68ccef",
	["Schurke"]				=	"|cfffff468",
	["Druide"]				=	"|cffff7c0a",
	["Schamane"]			=	"|cfff48cba",
	["Krieger"]				=	"|cffc69b6d",
} or GetLocale() == "frFR" and {
	["Chasseur"]			=	"|cffaad372",
	["D\195\169moniste"]	=	"|cff9382C9",
	["Pr\195\170tre"]		=	"|cffffffff",
	["Paladin"]				=	"|cfff48cba",
	["Mage"]				=	"|cff68ccef",
	["Voleur"]				=	"|cfffff468",
	["Druide"]				=	"|cffff7c0a",
	["Chaman"]				=	"|cfff48cba",
	["Guerrier"]			=	"|cffc69b6d",
} or {
	["Hunter"]				=	"|cffaad372",
	["Warlock"]				=	"|cff9382C9",
	["Priest"]				=	"|cffffffff",
	["Paladin"]				=	"|cfff48cba",
	["Mage"]				=	"|cff68ccef",
	["Rogue"]				=	"|cfffff468",
	["Druid"]				=	"|cffff7c0a",
	["Shaman"]				=	"|cfff48cba",
	["Warrior"]				=	"|cffc69b6d",
}

function AceTargetLog:Initialize()
	for i = 1,8,1 do
		UnitTable[i] = {Name = "", Level = "", Tag = "", Desc = ""}
	end
end

function AceTargetLog:Enable()
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	if not self:Get("BGColor") then 
		self:SetBGColor("0 0 0 1")
	else 
		self:SetBGColor(self:Get("BGColor"))
	end
	if not self:Get("Locked") then
		locked = false
	else
		locked = self:Get("Locked")
	end
end

function AceTargetLog:SetBGColor(colorString)
	local _,_ ,r,g,b,a = string.find(colorString, "(%d*%.?%d*)%s*(%d*%.?%d*)%s*(%d*%.?%d*)%s*(%d*%.?%d*)")
	if r and g and b and a then
		AceTargetLogFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
												tile = true, tileSize = 32,
												insets = { left = 0, right = 0, top = 0, bottom = 0 }})
		AceTargetLogFrame:SetBackdropColor(r,g,b,a)
		self:Set("BGColor", colorString)
	else
		self:Msg("Correct usage is: bgcolor <red> <green> <blue> <alpha>. Values between 0 and 1 are allowed.")
	end
end

function AceTargetLog:ShowFrame()
	if not AceTargetLogFrame:IsVisible() then AceTargetLogFrame:Hide() end
end

function AceTargetLog:HideFrame()
	if AceTargetLogFrame:IsVisible() then AceTargetLogFrame:Show() end
end

function AceTargetLog:UPDATE_MOUSEOVER_UNIT() 
	self:UpdateList("mouseover")
end
function AceTargetLog:PLAYER_TARGET_CHANGED()
	self:UpdateList("target")
end

function AceTargetLog:LockFrame()
	locked = not locked
	if not locked then self:Msg("The frame is now unlocked.") else self:Msg("The frame is now locked.")	end
	self:Set("Locked",locked)
end

function AceTargetLog:StartMove() if not locked then AceTargetLogFrame:StartMoving() end end 
function AceTargetLog:StopMove() if not locked then AceTargetLogFrame:StopMovingOrSizing() end end

function AceTargetLog:EntryOnClick(id)
	if UnitTable[id].Name and UnitTable[id].Name ~= "" then TargetByName(string.sub(UnitTable[id].Name,11,string.len(UnitTable[id].Name))) end
end

function AceTargetLog:UpdateList(checkType)
	local name = UnitName(checkType)
	if  name == nil or
		name == "Unknown Entity" or
		name == UnitName("player") or
		UnitIsDead(checkType) then return
	end
	local level	= UnitLevel(checkType)
	local guild, race, tag, color
	
	if UnitIsFriend(checkType,"player") then
		if UnitIsPlayer(checkType) then
			guild = GetGuildInfo(checkType)
			if UnitInParty(checkType) then 
				color = "Party"
			else
				if guild and guild == GetGuildInfo("player") then color = "Guild" else color = "FriendlyPC" end
			end
		else
			color = "FriendlyNPC"
		end
	elseif UnitIsEnemy(checkType,"player") then
		if UnitIsPlayer(checkType) then guild = GetGuildInfo(checkType) color = "EnemyPC" else color = "EnemyNPC" end
	else
		color = "NeutralNPC"
	end

	if UnitIsPlayer(checkType) then
		desc = ClassColors[UnitClass(checkType)]..UnitClass(checkType).."|r"
		tag = "|cffff4444P"
	else
		desc = UnitCreatureType(checkType)
		if UnitClassification(checkType) == "elite" then
			tag = "|cffff4444+"
		elseif string.find(UnitClassification(unit),"rare") then
			tag = "|cff4444ffR"
		elseif UnitClassification(checkType) == "worldboss" then
			tag = "|cffff4444B"
			level = -1
		else
			tag = ""
		end
	end

	if guild then desc = desc.."|r <"..ListColors[color]..guild.."|r>" end
	if level == -1 then level = "|cffff0000??" end
	name = ListColors[color]..name
	if type(tonumber(level)) == "number" then 
		level = self:GetLevelColor(tonumber(level))..level 
	end

	for i = 1, 8,1 do
		if UnitTable[i].Name == name and i > 1 then
		-- Name is already in the list, delete it, and move everything above one line down
			local j
			for j = i, 2, -1 do
				UnitTable[j].Level = UnitTable[j-1].Level
				UnitTable[j].Name = UnitTable[j-1].Name
				UnitTable[j].Tag = UnitTable[j-1].Tag
				UnitTable[j].Desc = UnitTable[j-1].Desc
				if j == 2 then table.remove(UnitTable,1) end
			end
		end
	end
	if UnitTable[1].Name == name then return end -- Name already at top, do nothing
	self:InsertNewEntry(level, tag, name, desc)
end

function AceTargetLog:InsertNewEntry(level, tag, name, desc)
	table.insert(UnitTable,1, {Level = level, Tag = tag, Name = name, Desc = desc})
	for i = 1,8,1 do
		if UnitTable[1].Level ~= nil then
			getglobal("AceTargetLogEntry"..i.."Level"):SetText(UnitTable[i].Level)
			getglobal("AceTargetLogEntry"..i.."Tag"):SetText(UnitTable[i].Tag)
			getglobal("AceTargetLogEntry"..i.."Name"):SetText(UnitTable[i].Name)
			getglobal("AceTargetLogEntry"..i.."Desc"):SetText(UnitTable[i].Desc)
		end
	end
end

function AceTargetLog:GetLevelColor(level)
	local diff = UnitLevel("Player") - level
	if 		diff > 7	then return "|cffafafaf"
	elseif	diff >= 3	then return "|cff00ff00"
	elseif	diff >= -2	then return "|cffffff00"
	elseif	diff >= -4 	then return "|cffff7f00"
	else return "|cffff0000"
	end
end

AceTargetLog:RegisterForLoad()

ace:RegisterFunctions(AceTargetLog, 
	{
		Msg  = function(self, ...) self.cmd:msg(unpack(arg)) end,
		Get  = function(self, var) return self.db:get(self.profilePath, var) end,
		Set  = function(self, var, val) self.db:set(self.profilePath, var, val) end,
	}
)
